﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDatabaseImplement.models;
namespace PizzeriaDatabaseImplement.Implements
{
    public class ImplementerStorage : IImplementerStorage
    {
        public List<ImplementerViewModel> GetFullList()
        {
            using var context = new PizzeriaDatabase();
            return context.Implementers
            .Select(x => x.GetViewModel)
           .ToList();
        }
        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO) &&
                string.IsNullOrEmpty(model.Password))
            {
                return new();
            }
            using var context = new PizzeriaDatabase();
            return context.Implementers
            .Where(x => (string.IsNullOrEmpty(model.ImplementerFIO) || x.ImplementerFIO.Contains(model.ImplementerFIO) &&
                         string.IsNullOrEmpty(model.Password) || x.ImplementerFIO.Contains(model.Password)))
           .Select(x => x.GetViewModel)
           .ToList();
        }
        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO) &&
                string.IsNullOrEmpty(model.Password) &&
                !model.Id.HasValue)
            {
                return null;
            }
            using var context = new PizzeriaDatabase();
            return context.Implementers
            .FirstOrDefault(x => (string.IsNullOrEmpty(model.ImplementerFIO) || x.ImplementerFIO == model.ImplementerFIO) &&
                                 (string.IsNullOrEmpty(model.Password) || x.Password == model.Password) &&
                                 (!model.Id.HasValue || x.Id == model.Id))
            ?.GetViewModel;
        }
        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            var newImplementer = Implementer.Create(model);
            if (newImplementer == null)
            {
                return null;
            }
            using var context = new PizzeriaDatabase();
            context.Implementers.Add(newImplementer);
            context.SaveChanges();
            return newImplementer.GetViewModel;
        }
        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var implementer = context.Implementers.FirstOrDefault(x => x.Id == model.Id);
            if (implementer == null)
            {
                return null;
            }
            implementer.Update(model);
            context.SaveChanges();
            return implementer.GetViewModel;
        }
        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var element = context.Implementers.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Implementers.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}